import os
import contextlib
import torch
import torch.nn as nn
import pandas as pd
import plotly.express as px
import shutil
import enum
from typing import *  # type: ignore
import __main__ as main
import subprocess
import sys
from torchvision.utils import save_image
from torchvision.io import read_image
import datetime